//========================================================================================
//
//	File:				Dialogs.h
//	Release Version:	$ ODF 2 $
//
//	Author:				Mary Boetcher
//
//	Copyright:			(c) 1993 - 1996 by Apple Computer, Inc., all rights reserved.
//
//========================================================================================

#ifndef DIALOGS_H
#define DIALOGS_H

// ----- Framework Includes -----

#ifndef FWFRAME_H
#include "FWFrame.h"
#endif

#ifndef FWEVENT_H
#include "FWEvent.h"
#endif

#ifndef FWDIALOG_H
#include "FWDialog.h"
#endif

//==============================================================================
// Forward Declarations
//==============================================================================

class CIntlTestPart;
class CMyEditView;
class FW_CStaticText;

//==============================================================================
// class COpTestDialogFrame
//==============================================================================

class COpTestDialogFrame : public FW_CDialogFrame
{
  public:
	FW_DECLARE_AUTO(COpTestDialogFrame)

	COpTestDialogFrame(Environment* ev, 
				ODFrame* odFrame, 
				FW_CPresentation* presentation, 
				CIntlTestPart* part);

	virtual ~COpTestDialogFrame();

  public:	
	// ---- FW_CDialogFrame API
	virtual void 		HandleNotification(Environment* ev, const FW_CNotification& notification);

	// ----- FW_CView & FW_CFrame API
	virtual void 		PostCreateViewFromStream(Environment* ev);
  
	// -----New API
	void 				Initialize(Environment* ev, CMyEditView* editView);

	// ----- Private data
  private:
  	CIntlTestPart* 		fTestPart;
	FW_CString			fTargetString;
	FW_Boolean 			fIsJapanese;
	FW_CStaticText*		fResultView;		// static text where result is displayed
};

//==============================================================================
// class CReaderTestDialogFrame
//==============================================================================

class CReaderTestDialogFrame : public FW_CDialogFrame
{
  public:
	FW_DECLARE_AUTO(CReaderTestDialogFrame)

	CReaderTestDialogFrame(Environment* ev, 
				ODFrame* odFrame, 
				FW_CPresentation* presentation, 
				CIntlTestPart* part);

	virtual ~CReaderTestDialogFrame();

  public:	
	// ---- FW_CDialogFrame API
	virtual void 		HandleNotification(Environment* ev, const FW_CNotification& notification);

	// ----- FW_CView & FW_CFrame API
	virtual void 		PostCreateViewFromStream(Environment* ev);
  
	// -----New API
	void 				Initialize(Environment* ev, CMyEditView* editView);

  private:
	void ClearResultViews(Environment* ev);
	void UpdateDisplay(Environment* ev, FW_LChar ch);

	void GetCharacterAndAdvanceReader(Environment* ev);
	void BackupReaderAndGetCharacter(Environment* ev);
	void AdvanceReader(Environment* ev);
	void BackupReader(Environment* ev);
	void PeekCharReader(Environment* ev);

	// ----- Private data
  private:
  	CIntlTestPart* 		fTestPart;
	FW_CString			fTargetString;
	FW_Boolean 			fIsJapanese;
	FW_CStaticText*		fCharacterView;		// static text where character is displayed
	FW_CStaticText*		fPositionView;		// static text where byte position is displayed
	FW_CStringReader*	fReader;
	FW_ByteCount		fBytePosition;
};

//==============================================================================
// class CNumberTestDialogFrame
//==============================================================================

class CNumberTestDialogFrame : public FW_CDialogFrame
{
  public:
	FW_DECLARE_AUTO(CNumberTestDialogFrame)

	CNumberTestDialogFrame(Environment* ev, 
				ODFrame* odFrame, 
				FW_CPresentation* presentation, 
				CIntlTestPart* part);

	virtual ~CNumberTestDialogFrame();

  public:	
	// ---- FW_CDialogFrame API
	virtual void 		HandleNotification(Environment* ev, const FW_CNotification& notification);

	// ----- FW_CView & FW_CFrame API
	virtual void 		PostCreateViewFromStream(Environment* ev);
  
	// -----New API
	void 				Initialize(Environment* ev, FW_Boolean testNewCode);

	void				SetInputFont(Environment* ev, FW_Boolean setJapanese);

	// ----- Private data
  private:
  	CIntlTestPart* 		fTestPart;
	FW_CString			fTargetString;
	FW_Boolean 			fIsNewTest;
	FW_Boolean 			fIsJapanese;
	FW_CStaticText*		fResultView;		// static text where result is displayed
};


#endif