//========================================================================================
//
//	File:				EditViews.cpp
//	Release Version:	$ ODF 2 $
//
//	Author:				Mary Boetcher
//
//	Copyright:			(c) 1993 - 1996 by Apple Computer, Inc., all rights reserved.
//
//========================================================================================

#include "IntlTest.hpp"

#ifndef EDITVIEWS_H
#include "EditViews.h"
#endif

#ifndef DEFINES_K
#include "Defines.k"
#endif

#ifndef COMMANDS_H
#include "Commands.h"
#endif

// for gJapaneseLocale
#ifndef PART_H
#include "Part.h"
#endif

// ----- Part Layer -----

#ifndef FWEDVIEW_H
#include "FWEdView.h"
#endif

#ifndef FWEVENT_H
#include "FWEvent.h"
#endif

#ifndef FWCONTXT_H
#include "FWContxt.h"
#endif

#ifndef FWFRAME_H
#include "FWFrame.h"
#endif

#ifndef FWPART_H
#include "FWPart.h"
#endif

// ----- OS Layer -----

#ifndef FWMENU_H
#include "FWMenu.h"
#endif

// ----- Foundation Layer -----

#ifndef FWINTERE_H
#include "FWIntere.h"
#endif

// ----- Macintosh Includes -----

#ifndef __SCRIPT__
#include <Script.h>
#endif

#ifndef __GESTALT__
#include <Gestalt.h>
#endif

#ifndef __AEPACKOBJECT__
#include <AEPackObject.h>
#endif

//========================================================================================
// RunTime Info
//========================================================================================

#ifdef FW_BUILD_MAC
#pragma segment odfIntlTest
#endif

//========================================================================================
// Constants and Globals

#define cTypingCommand 110
#define cTSMTypingCommand 111

// Default Japanese font
FW_CFont gFontOsaka(FW_CString32("Osaka"), FW_kPlain, FW_IntToFixed(14));

// Font and Size menus
ODMenuID gSizeMenuID = 0;					// Size menu's ID
FW_CPullDownMenu* gFontMenu = NULL;			// current Font menu
short gFontCount = 0;						// number of items in the current Font menu
FW_CPullDownMenu* gDefaultFontMenu = NULL;	// default Font menu (Roman fonts only)
FW_CPullDownMenu* gJFontMenu = NULL;		// Japanese Font menu

//----------------------------------------------------------------------------------------
// Pre- and post-update callback procs for TSMTE

pascal void MyTSMTEPreUpdateProc(TEHandle textH, long refCon);

pascal void MyTSMTEPostUpdateProc(TEHandle textH, 
								  long fixLen, long inputAreaStart, long inputAreaEnd, 
								  long pinStart, long pinEnd, long refCon);

static TSMTEPreUpdateUPP gTSMTEPreUpdateUPP = NULL;
static TSMTEPostUpdateUPP gTSMTEPostUpdateUPP = NULL;

//========================================================================================
// TSMTE callback routines
//========================================================================================

//----------------------------------------------------------------------------------------
pascal void MyTSMTEPreUpdateProc(TEHandle textH, long refCon)
{
	FW_UNUSED(textH);

	FW_TRY
	{
		CJEditView* editView = (CJEditView*)refCon;	/* !!! */
	
		if (editView->IsFirstEventInSequence())
		{
			//--- Make a new typing command and save the Undo state
			editView->DoMakeTypingCommand();
		}
	}
	FW_CATCH_BEGIN
	FW_CATCH_EVERYTHING()
	{
		// do nothing; no way to return an error
	}
	FW_CATCH_END
}

//----------------------------------------------------------------------------------------
pascal void MyTSMTEPostUpdateProc(TEHandle textH, 
								  long fixLen, long inputAreaStart, long inputAreaEnd, 
								  long pinStart, long pinEnd, long refCon)
{
	// fixLen is the length of the confirmed text in bytes.
	FW_UNUSED(inputAreaStart);
	FW_UNUSED(inputAreaEnd);
	FW_UNUSED(pinStart);
	FW_TRY
	{
		if (fixLen > 0)		// some characters were confirmed
		{
			CJEditView* editView = (CJEditView*)refCon;	/* !!! */
			Environment* ev = somGetGlobalEnvironment();

			//--- Add the confirmed characters to the typing command
			char* p = (char*) (*((**textH).hText)) + pinEnd - fixLen;
			CTSMInput input(p, fixLen);
			editView->AddInput(ev, &input);
		}
	}
	FW_CATCH_BEGIN
	FW_CATCH_EVERYTHING()
	{
		// do nothing; no way to return an error
	}
	FW_CATCH_END
}

//========================================================================================
// FW_MTSMHandler
//========================================================================================

FW_DEFINE_CLASS_M0(FW_MTSMHandler)

//----------------------------------------------------------------------------------------
FW_MTSMHandler::FW_MTSMHandler(FW_CEditView* theView)
{
	fEditView = theView;
	fTSMDocId = NULL;
	fTSMHandle = NULL;
	fError = noErr;
}

//----------------------------------------------------------------------------------------
FW_MTSMHandler::~FW_MTSMHandler()
{
	if (fTSMDocId)
		fError = ::DeleteTSMDocument(fTSMDocId);
	fTSMHandle = NULL;
	fEditView = NULL;

	// Dispose any UPPs we made
	if (gTSMTEPreUpdateUPP)
	{
		DisposeRoutineDescriptor(gTSMTEPreUpdateUPP);
		gTSMTEPreUpdateUPP = NULL;
	}
	if (gTSMTEPostUpdateUPP)
	{
		DisposeRoutineDescriptor(gTSMTEPostUpdateUPP);
		gTSMTEPostUpdateUPP = NULL;
	}
}

//----------------------------------------------------------------------------------------
FW_Boolean FW_MTSMHandler::CreateTSMDocument(TEHandle te)
{
	FW_Boolean result = false;

	// Check for the presence of TSMTE
	long gestaltResponse;
	ODBoolean hasTSMTE = false;
	if (Gestalt(gestaltTSMTEAttr, &gestaltResponse) == noErr)
		hasTSMTE = (gestaltResponse & (1<<gestaltTSMTEPresent) != 0);

	if (hasTSMTE)	// Don't create a TSM document if TSMTE is not present
	{
		TSMDocumentID id = NULL;
		InterfaceTypeList supportedTypes;
		supportedTypes[0] = kTSMTEInterfaceType;
		TSMTERecHandle tsmHandle;
	
		fError = ::NewTSMDocument(1, supportedTypes, &id, (long) &tsmHandle);
		if (fError == noErr)
		{
			// Create UPPs for pre- and post-update callbacks
			gTSMTEPreUpdateUPP = NewTSMTEPreUpdateProc(&MyTSMTEPreUpdateProc);
			gTSMTEPostUpdateUPP = NewTSMTEPostUpdateProc(&MyTSMTEPostUpdateProc);

			// Initialize the TSM record
			fTSMDocId = id;
			fTSMHandle = tsmHandle;
			(*tsmHandle)->textH = te;
			(*tsmHandle)->preUpdateProc = gTSMTEPreUpdateUPP;
			(*tsmHandle)->postUpdateProc = gTSMTEPostUpdateUPP;
			(*tsmHandle)->updateFlag = kTSMTEAutoScroll;
			(*tsmHandle)->refCon = (long)fEditView; /* !!! */
			result = true;
		}
	}

	// Copied from Sample TSMTE DragText
	::UseInputWindow( kODNULL, false);

	return result;
}

//----------------------------------------------------------------------------------------
FW_Boolean FW_MTSMHandler::DoActivateTSM(FW_Boolean becomingActive)
{
	FW_Boolean result = false;

	if (fTSMDocId)
	{
		if (becomingActive)
		{
			fError = ::ActivateTSMDocument(fTSMDocId);

			// Copied from Sample TSMTE DragText
			::UseInputWindow( kODNULL, false);
		}
		else
			fError = ::DeactivateTSMDocument(fTSMDocId);
		if (fError == noErr)
			result = true;
	}

	return result;
}

//----------------------------------------------------------------------------------------
FW_Boolean FW_MTSMHandler::DoFixTSMDocument()
{
	FW_Boolean result = false;

	if (fTSMDocId)
	{
		fError = ::FixTSMDocument(fTSMDocId);
		if (fError == noErr)
			result = true;
	}

	return result;
}

//========================================================================================
// CJEditView - for editing Japanese script
//========================================================================================

FW_DEFINE_AUTO(CJEditView)

FW_DEFINE_CLASS_M2(CJEditView, CMyEditView, FW_MTSMHandler);

const FW_ClassTypeConstant LMyJEditView = FW_TYPE_CONSTANT('M','y','j','v');
FW_REGISTER_ARCHIVABLE_CLASS(LMyJEditView, CJEditView, CJEditView::Create, FW_CView::Read, CJEditView::Destroy, FW_CView::Write)

//----------------------------------------------------------------------------------------
CJEditView::CJEditView(Environment* ev,
						FW_CSuperView* container, 
						const FW_CRect& bounds,
						ODID viewId,
						const FW_CFont& font,
						short maxChars,
						unsigned short attributes)
:	CMyEditView(ev, container, bounds, viewId, font, maxChars, attributes),
	FW_MTSMHandler(this),
	fTSMTypingCommand(NULL),
	fMyFont(0),
	fMyFontScript(smJapanese),
	fPreviousFontScript(0)
{
	// Save the font number
	fMyFont = font.MacGetFontID();
	FW_END_CONSTRUCTOR
}

//----------------------------------------------------------------------------------------
CJEditView::CJEditView(Environment* ev)
	: CMyEditView(ev),
	FW_MTSMHandler(this),
	fTSMTypingCommand(NULL),
	fMyFont(0),
	fMyFontScript(smJapanese),
	fPreviousFontScript(0)
{
	FW_END_CONSTRUCTOR
}

//----------------------------------------------------------------------------------------
CJEditView::~CJEditView()
{
	FW_START_DESTRUCTOR
}

//----------------------------------------------------------------------------------------
void CJEditView::PostCreateViewFromStream(Environment* ev)
{
	CMyEditView::PostCreateViewFromStream(ev);
	InitJEditView(ev);
}

//----------------------------------------------------------------------------------------
//	CJEditView Archiving methods
//----------------------------------------------------------------------------------------

void* CJEditView::Create(FW_CReadableStream& stream, FW_ClassTypeConstant type)
{
FW_UNUSED(stream);
FW_UNUSED(type);
	FW_SOMEnvironment ev;
	return FW_NEW(CJEditView, (ev));
}

//----------------------------------------------------------------------------------------
void CJEditView::Destroy(void* object, FW_ClassTypeConstant type)
{
FW_UNUSED(type);
	CJEditView* self = (CJEditView*) object;
	delete self;
}

//----------------------------------------------------------------------------------------
// CJEditView::ActivateTarget
//----------------------------------------------------------------------------------------
void CJEditView::ActivateTarget(Environment* ev, FW_Boolean tabSelection)
{
	// Save the user's keyboard script
	fPreviousFontScript = ::GetScriptManagerVariable(smKeyScript);

	// Activate TSM
	this->DoActivateTSM(true);

	// Set the keyboard script to match my font's script
	if (fMyFontScript != fPreviousFontScript)
		::KeyScript(fMyFontScript);

	// Call inherited
	CMyEditView::ActivateTarget(ev, tabSelection);
}

//----------------------------------------------------------------------------------------
// CJEditView::DeactivateTarget
//----------------------------------------------------------------------------------------
void CJEditView::DeactivateTarget(Environment* ev)
{
	// Terminate active input
	this->DoFixTSMDocument();

	// Deactivate TSM
	this->DoActivateTSM(false);

	// Call inherited
	CMyEditView::DeactivateTarget(ev);

	// Restore the user's keyboard script
	::KeyScript(fPreviousFontScript);
}

//----------------------------------------------------------------------------------------
// CJEditView::DoMenu
//----------------------------------------------------------------------------------------
FW_Handled CJEditView::DoMenu(Environment* ev, const FW_CMenuEvent& theMenuEvent) // Override
{
	FW_Handled menuHandled = FW_kHandled;
	ODCommandID id = theMenuEvent.GetCommandID(ev);
	
	if ((cFirstFontCommand <= id && id < cFirstFontCommand+gFontCount) ||
		(cFontSize9 <= id && id < cFontSize9 + 5))
	{
		// Call inherited to change the font
		return CMyEditView::DoMenu(ev, theMenuEvent);
	}

	switch (id)
	{
		case kODCommandCut:
		case kODCommandCopy:
		case kODCommandPaste:
		case kODCommandClear:
		case kODCommandSelectAll: {
			menuHandled = CMyEditView::DoMenu(ev, theMenuEvent);
			this->DoFixTSMDocument();
			break;
		}

		default:
			menuHandled = FW_CEditView::DoMenu(ev, theMenuEvent);
	}

	return menuHandled;
}

//----------------------------------------------------------------------------------------
void CJEditView::InitJEditView(Environment* ev)
{
	// Create a TSM document, if TSMTE is present
	this->CreateTSMDocument((TEHandle)GetPlatformEditHandle(ev));

	// Set the font to Osaka - already done in constructor
//	FW_CEditView::SetFont(ev, gFontOsaka);
}

//-----------------------------------------------------------------------------------------
void CJEditView::AddInput(Environment* ev, CTSMInput* input)
{
	FW_ASSERT(fTSMTypingCommand);
	fTSMTypingCommand->AddInput(ev, input);
}

//-----------------------------------------------------------------------------------------
FW_Boolean CJEditView::IsFirstEventInSequence()
{
	if (fTSMTypingCommand == NULL) return true;
	return fTSMTypingCommand->IsCompleted();
}

//-----------------------------------------------------------------------------------------
void CJEditView::DoMakeTypingCommand(Environment* ev)
{
	// If called by a callback routine, there's no SOM variable
	if (ev == NULL)
		ev = somGetGlobalEnvironment();

	this->DoneTyping(ev);	// this was already called, presumably...

	fTSMTypingCommand = NewTypingCommand(ev, NULL);	// no input yet
	fTypingCommand->Execute(ev, false);				// calls DoIt, which calls SaveUndoState
}

//-----------------------------------------------------------------------------------------
CTSMTypingCommand* CJEditView::NewTypingCommand(Environment* ev, CTSMInput* firstInput)
{
	CTSMTypingCommand* cmd = FW_NEW(CTSMTypingCommand, (ev, cTSMTypingCommand, GetFrame(ev),
														this, firstInput));
	fTypingCommand = cmd;
	return cmd;
}

//-----------------------------------------------------------------------------------------
FW_Handled CJEditView::DoCharKey(Environment* ev, const FW_CCharKeyEvent& event)
{
	// Ignore Command-characters which are not associated with menu commands
	if (event.IsItemModifier(ev))
		return FW_kNotHandled;

	char ch = event.GetChar(ev);

	if (IsArrowKey(ch))			// arrow keys
	{
		this->DoneTyping(ev);
	}
	else if (ch == chBackspace)
	{
		if (this->IsFirstEventInSequence())
		{
			//--- Make a new typing command and save the Undo state
			this->DoMakeTypingCommand(ev);
		}
		fTSMTypingCommand->Backspace(ev);
	}
	else	// user typed a single-byte character, such as a CR
	{
		//--- Make sure we have room for the character
		if (PrivCheckMaxChars(1))
		{
			FW_Beep();
			return FW_kHandled;
		}

		if (ch == chReturn)	// CR
		{
			this->DoneTyping(ev);
		}

		//--- Pass character to the typing command
		CTSMInput input(ch);
		if (fTSMTypingCommand && fTSMTypingCommand->IsNotCompleted())
			fTSMTypingCommand->AddInput(ev, &input);
		else
		{
			fTSMTypingCommand = NewTypingCommand(ev, &input);
			fTSMTypingCommand->Execute(ev, false);		// DoIt; don't delete if not undoable
		}
	}

	//--- Tell TextEdit to handle the character
	FW_CEditView::InputCharKey(ev, ch);
	return FW_kHandled;
}

//----------------------------------------------------------------------------------------
FW_CPullDownMenu* CJEditView::GetMyFontMenu()
{
	return gJFontMenu;
}

//========================================================================================
// CMyEditView - for editing 1-byte scripts
//========================================================================================

FW_DEFINE_AUTO(CMyEditView)

FW_DEFINE_CLASS_M1(CMyEditView, FW_CEditView);

const FW_ClassTypeConstant LMyEditView = FW_TYPE_CONSTANT('M','y','e','v');
FW_REGISTER_ARCHIVABLE_CLASS(LMyEditView, CMyEditView, CMyEditView::Create, FW_CView::Read, CMyEditView::Destroy, FW_CView::Write)

//----------------------------------------------------------------------------------------
//	CMyEditView constructors, destructor
//----------------------------------------------------------------------------------------
CMyEditView::CMyEditView(Environment* ev,
						 FW_CSuperView* container,
						 const FW_CRect& bounds,
						 ODID viewId,
						 const FW_CFont& font,
						 short maxChars,
						 unsigned short attributes)
:	FW_CEditView(ev, container, bounds, viewId, "", font, maxChars, attributes),
	fTypingCommand(NULL),
	fFontCmd(0)
{
	font.GetFontName(fFontName);
	fFontSize = FW_FixedToInt(font.GetFontSize());
	FW_END_CONSTRUCTOR
}

//----------------------------------------------------------------------------------------
CMyEditView::CMyEditView(Environment* ev)
	: FW_CEditView(ev),
	fTypingCommand(NULL),
	fFontCmd(0)
{
	FW_END_CONSTRUCTOR
}

//----------------------------------------------------------------------------------------
CMyEditView::~CMyEditView()
{
	FW_START_DESTRUCTOR
}

//----------------------------------------------------------------------------------------
//	CMyEditView Archiving methods
//----------------------------------------------------------------------------------------

void* CMyEditView::Create(FW_CReadableStream& stream, FW_ClassTypeConstant type)
{
FW_UNUSED(stream);
FW_UNUSED(type);
	FW_SOMEnvironment ev;
	return FW_NEW(CMyEditView, (ev));
}

//----------------------------------------------------------------------------------------
void CMyEditView::Destroy(void* object, FW_ClassTypeConstant type)
{
FW_UNUSED(type);
	CMyEditView* self = (CMyEditView*) object;
	delete self;
}

//----------------------------------------------------------------------------------------
void CMyEditView::Flatten(Environment* ev, FW_CWritableStream& archive) const
{
	FW_CEditView::Flatten(ev, archive);
}

//----------------------------------------------------------------------------------------
void CMyEditView::InitializeFromStream(Environment* ev, FW_CReadableStream& stream)
{
	FW_CEditView::InitializeFromStream(ev, stream);
}

//----------------------------------------------------------------------------------------
void CMyEditView::PostCreateViewFromStream(Environment* ev)
{
	FW_CFont font = this->GetFont(ev);
	font.GetFontName(fFontName);
	fFontSize = FW_FixedToInt(font.GetFontSize());
	fFontCmd = GetCurrentFontCmd(ev);
}

//----------------------------------------------------------------------------------------
//	CMyEditView methods overriding FW_CEditView
//-----------------------------------------------------------------------------------------

FW_Handled CMyEditView::DoCharKey(Environment* ev, const FW_CCharKeyEvent& event)
{
	// Ignore Command-characters which are not associated with menu commands
	if (event.IsItemModifier(ev))
		return FW_kNotHandled;

	char ch = event.GetChar(ev);

	if (IsArrowKey(ch))			// arrow keys
	{
		this->DoneTyping(ev);
	}
	else
	{
		if (ch != chBackspace)
		{
			//--- Make sure we have room for the character
			if (PrivCheckMaxChars(1))
			{
				FW_Beep();
				return FW_kHandled;
			}
	
			if (ch == FW_kPlatformNewLineChar)	// CR
			{
				this->DoneTyping(ev);
			}
		}

		//--- Pass character to the typing command
		if (fTypingCommand && fTypingCommand->IsNotCompleted())
			fTypingCommand->AddCharacter(ev, ch);
		else
		{
			fTypingCommand = NewTypingCommand(ev, ch);
			fTypingCommand->Execute(ev, false);	// don't delete if not undoable
		}
	}

	//--- Tell TextEdit to handle the character
	FW_CEditView::InputCharKey(ev, ch);
	return FW_kHandled;
}

//-----------------------------------------------------------------------------------------
FW_Handled CMyEditView::DoMouseDown(Environment* ev, const FW_CMouseEvent& event)
{
	this->DoneTyping(ev);
	return FW_CEditView::DoMouseDown(ev, event);
}

//----------------------------------------------------------------------------------------
void CMyEditView::ActivateTarget(Environment* ev, FW_Boolean tabSelection)
{
	FW_ASSERT(gFontMenu != NULL);
	if (gFontMenu != GetMyFontMenu())	// time to swap Font menus
	{
		// Switch to the default Font menu
		FW_CMenuBar* menuBar = GetMenuBar(ev);
		menuBar->DetachMenu(ev, gFontMenu);

		FW_CPullDownMenu* sizeMenu = menuBar->FindMenuWithID(ev, gSizeMenuID);
		FW_ASSERT(sizeMenu);
		menuBar->AdoptMenuBefore(ev, GetMyFontMenu(), sizeMenu);
		
		gFontMenu = GetMyFontMenu();
		gFontCount = gFontMenu->CountItems(ev);
		/* Check the menu id; could it reach 255? */
		ODMenuID id = gFontMenu->GetMenuID(ev);
		FW_ASSERT(id < 255);
	}

	// Call inherited
	FW_CEditView::ActivateTarget(ev, tabSelection);
}

//----------------------------------------------------------------------------------------
FW_Handled CMyEditView::DoAdjustMenus(Environment* ev, FW_CMenuBar* menuBar, 
									   FW_Boolean hasMenuFocus, FW_Boolean isRoot)
{
	if (hasMenuFocus)
	{
		//-- Enable all items in the Font menu
		ODCommandID c = cFirstFontCommand;
		for (short i=0; i < gFontCount; c++, i++)
			menuBar->EnableAndCheckCommand(ev, c, true, false);

		//-- Enable all items in the Size menu
		c = cFontSize9;
		for (short j=0; j < 5; c++, j++)
			menuBar->EnableAndCheckCommand(ev, c, true, false);

		//-- Put a check in front of the current font and size
		c = GetCurrentFontCmd(ev);
		if (c != 0)
			menuBar->CheckCommand(ev, c, true);
		menuBar->CheckCommand(ev, GetCurrentSizeCmd(), true);
	}
	
	return FW_CEditView::DoAdjustMenus(ev, menuBar, hasMenuFocus, isRoot);
}

//----------------------------------------------------------------------------------------
FW_Handled CMyEditView::DoMenu(Environment* ev, const FW_CMenuEvent& theMenuEvent) // Override
{
	FW_Handled menuHandled = FW_kHandled;
	ODCommandID id = theMenuEvent.GetCommandID(ev);
	
	if (cFirstFontCommand <= id && id < cFirstFontCommand+gFontCount)
	{
		CFontCommand* cmd = FW_NEW(CFontCommand, (ev, id, GetFrame(ev), this));
		if (cmd)
			cmd->Execute(ev);
		return menuHandled;
	}

	if (cFontSize9 <= id && id < cFontSize9 + 5)
	{
		CFontCommand* cmd = FW_NEW(CFontCommand, (ev, id, GetFrame(ev), this));
		if (cmd)
			cmd->Execute(ev);
		return menuHandled;
	}

	switch (id)
	{
		case kODCommandCut: 
		case kODCommandCopy: 
		case kODCommandPaste: 
		case kODCommandClear:
		{	
			this->DoneTyping(ev);
			// Create and execute one of our undoable clipboard commands
			FW_CClipboardCommand* cmd = GetFrame(ev)->NewClipboardCommand(ev, id);
			if (cmd)
				cmd->Execute(ev);
			break;
		}

		case kODCommandSelectAll:
			this->DoneTyping(ev);
			// fall thru to default case

		default:
			menuHandled = FW_CEditView::DoMenu(ev, theMenuEvent);	// for Select All or whatever
	}

	return menuHandled;
}

//----------------------------------------------------------------------------------------
void CMyEditView::DeactivateTarget(Environment* ev)
{
	// finish the typing command
	this->DoneTyping(ev);

	// call inherited
	FW_CEditView::DeactivateTarget(ev);
}

//-----------------------------------------------------------------------------------------
void CMyEditView::DoneTyping(Environment* ev)
{
	if (fTypingCommand)
	{
		fTypingCommand->CompleteTyping(ev);
	}
}

//-----------------------------------------------------------------------------------------
void CMyEditView::TypingCommandAboutToBeDestroyed(CTypingCommand* cmd)
{
	// This routine is called when the typing command is destroyed by DisposeActionState
	if (fTypingCommand)
	{
		if (fTypingCommand == cmd)
			fTypingCommand = NULL;
	}
}

//-----------------------------------------------------------------------------------------
FW_Boolean CMyEditView::IsArrowKey(char ch)
{
	switch (ch)
	{
		case chLeft:
		case chRight:
		case chUp:
		case chDown:
			return true;
	}
	return false;
}

//-----------------------------------------------------------------------------------------
CTypingCommand* CMyEditView::NewTypingCommand(Environment* ev, char firstChar)
{
	CTypingCommand* cmd = FW_NEW(CTypingCommand, (ev, cTypingCommand, GetFrame(ev),
												  this, true, firstChar));
	return cmd;
}

//----------------------------------------------------------------------------------------
void CMyEditView::CommandToString(Environment* ev, ODCommandID id, FW_CString& string)
{
	GetMenuBar(ev)->GetItemString(ev, id, string);
}

//----------------------------------------------------------------------------------------
FW_CMenuBar* CMyEditView::GetMenuBar(Environment* ev)
{
	return GetFrame(ev)->GetPart(ev)->GetMenuBar(ev);
}

//----------------------------------------------------------------------------------------
void CMyEditView::DoSetFont(Environment* ev, ODCommandID id, const FW_CString& fontName)
{
	this->SetFont(ev, FW_CFont(fontName, FW_kPlain, FW_IntToFixed(fFontSize)));
	fFontName = fontName;
	fFontCmd = id;
	this->Invalidate(ev);	// Invalidate this view
}

//----------------------------------------------------------------------------------------
void CMyEditView::DoSetFontSize(Environment* ev, int fontSize)
{
	this->SetFont(ev, FW_CFont(fFontName, FW_kPlain, FW_IntToFixed(fontSize)));
	fFontSize = fontSize;
	this->Invalidate(ev);	// Invalidate this view
}

//----------------------------------------------------------------------------------------
FW_CFont CMyEditView::GetCurrentFont()
{
	return FW_CFont(fFontName, FW_kPlain, FW_IntToFixed(fFontSize));
}

//----------------------------------------------------------------------------------------
ODCommandID CMyEditView::GetCurrentSizeCmd()
{
	ODCommandID result = cFontSize12;
	switch (fFontSize)
	{
		case 9:		result = cFontSize9; break;
		case 10:	result = cFontSize10; break;
		case 14:	result = cFontSize14; break;
		case 18:	result = cFontSize18; break;
	}
	return result;
}

//----------------------------------------------------------------------------------------
ODCommandID CMyEditView::GetCurrentFontCmd(Environment* ev)
{
	ODCommandID result = fFontCmd;
	if (result == 0)
	{
		// Find out what the current font cmd is
		ODCommandID c = cFirstFontCommand;
		FW_CMenuBar* menuBar = GetMenuBar(ev);
		FW_CString fontName;
		for (short i=0; i < gFontCount; i++, c++)
		{
			menuBar->GetItemString(ev, c, fontName);
/*	Problem comparing fontname strings: the string returned by GetItemString
	apparently has its fLangCode field set to 0 even if the fScriptCode is not 0.
	Workaround by comparing only the characters, ignoring their locales.
			if (fontName == fFontName)
*/
			if (strncmp(fontName.RevealBuffer(), fFontName.RevealBuffer(), fontName.GetByteLength()) == 0)
			{
				result = c;
				break;
			}
		}
		fFontCmd = result;
	}
	return result;	// could be 0
}

//----------------------------------------------------------------------------------------
FW_CPullDownMenu* CMyEditView::GetMyFontMenu()
{
	return gDefaultFontMenu;
}
