//========================================================================================
//
//	File:				Part.h
//	Release Version:	$ ODF 2 $
//
//	Copyright:			(c) 1993 - 1996 by Apple Computer, Inc., all rights reserved.
//
//========================================================================================

#ifndef PART_H
#define PART_H

#ifndef DEFINES_K
#include "Defines.k"
#endif

// ----- Framework Includes -----

#ifndef FWPART_H
#include "FWPart.h"
#endif

// ----- OS Layer -----

#ifndef FWRESFIL_H
#include "FWResFil.h"
#endif

// ----- Foundation Layer -----

#ifndef FWSTDDEF_H
#include "FWStdDef.h"
#endif

#ifndef FWBNDSTR_H
#include "FWBndStr.h"
#endif


//========================================================================================
// Forward Declarations
//========================================================================================

class CIntlTestContent;
class CIntlTestFrame;
class CEditView;
class CIntlTestSelection;
class CMyEditView;

//========================================================================================
// CIntlTestPart
//========================================================================================

class CIntlTestPart : public FW_CPart
{
  public:
	FW_DECLARE_AUTO(CIntlTestPart)
	
//----------------------------------------------------------------------------------------
//	Initialization/Destruction
//
  public:
	CIntlTestPart(ODPart* odPart);
	virtual ~CIntlTestPart();

	virtual void Initialize(Environment* ev, ODStorageUnit* storageUnit, FW_Boolean fromStorage);

//----------------------------------------------------------------------------------------
//	Inherited API
//
  public:
	virtual FW_CContent* NewPartContent(Environment* ev);

	virtual FW_CFrame*	NewFrame(Environment* ev,
								 ODFrame* odFrame,
								 FW_CPresentation* presentation,
								 FW_Boolean fromStorage);

	virtual FW_Handled	DoAbout(Environment* ev);

	virtual FW_Handled	DoAdjustMenus(Environment* ev,
									  FW_CMenuBar* menuBar, 
									  FW_Boolean hasMenuFocus,
									  FW_Boolean isRoot);

	virtual FW_Handled	DoMenu(Environment* ev, const FW_CMenuEvent& theMenuEvent);

//----------------------------------------------------------------------------------------
//	New API
//
  public:
	CIntlTestContent*	GetPartContent();
	CIntlTestSelection*	GetTestSelection();
	ODID				WhichEditViewIsActive(Environment* ev);

	void				UpdateTextData(Environment* ev);
	void				UpdateViewsFromTextData(Environment* ev);

  private:
	void				InitializeMenus(Environment* ev);

	void				OpenOperatorTestDialog(Environment* ev);
	void				OpenReaderTestDialog(Environment* ev);
	void				OpenNumberTestDialog(Environment* ev, ODCommandID id);

	CMyEditView*		GetActiveEditView(Environment* ev);

//----------------------------------------------------------------------------------------
//	Data Members
//
  private:
	CIntlTestContent*	fPartContent;
	CIntlTestSelection*	fTestSelection;
	FW_CPresentation*	fMainPresentation;
	FW_CPresentation*	fOpTestPresentation;
	FW_CPresentation*	fReaderTestPresentation;
	FW_CPresentation*	fNumberTestPresentation;
	CIntlTestFrame*		fTestFrame;
};

//----------------------------------------------------------------------------------------
inline CIntlTestContent* CIntlTestPart::GetPartContent()
{
	return fPartContent;
}

//----------------------------------------------------------------------------------------
inline CIntlTestSelection* CIntlTestPart::GetTestSelection()
{
	return fTestSelection;
}

//========================================================================================
extern FW_CString MyLoadRStringByID(Environment* ev,
							 FW_PResourceFile& file,
							 FW_ResourceID resourceId,
							 FW_ResourceType resourceType);

extern FW_Locale gJapaneseLocale;


#endif
