//========================================================================================
//	File:				Views.fr
//	Release Version:	$ ODF 2 $
//
//	Contains:			Resources defining the IntlTest part's views
//
//	Copyright:			(c) 1993 - 1996 by Apple Computer, Inc., all rights reserved.
//========================================================================================

#ifndef FWRESFIL_K
#include "FWResFil.k"
#endif

#ifndef FWVIEWS_FR
#include "FWViews.fr"
#endif

#ifndef FWNOTDEF_H
#include "FWNotDef.h"	 
#endif

#ifndef DEFINES_K
#include "Defines.k"
#endif

#ifndef INTLRES_K
#include "IntlRes.k"	// for RJStringData
#endif

//========================================================================================
#define JAPANESE_FONT	{ FW_FIX(14), FW_kPlain, "osaka" }	// Osaka font for Japanese text

//========================================================================================
// Custom edit views

type RMyEditView : FW_REditView(Label='Myev')
{
};

type RJEditView : RMyEditView(Label='Myjv')
{
};

//========================================================================================
// resource FW_RFrameLayout(kIntlTestView)
//========================================================================================
// See constants in "FWViews.k", macros & resource types in "FWViews.fr"

resource FW_RFrameLayout(kIntlTestView)
{
	{FW_FIX(320),FW_FIX(260)},		// LayoutSize (HV)
	
	{						// Start list of frame's subviews
		FW_RStaticText
		(
			0,							// view id
			{ FW_FIX(10),FW_FIX(10),FW_FIX(300),FW_FIX(30)},	// LTRB
			FW_kFixedBounds,			// binding
			FW_NORMAL_FONT, 
			FW_RGB_WHITE,				// erase color
			"Type here:"			
		),
		RMyEditView						// edit view for English
		(
			kEnglishEditView,			// view id
			{ FW_FIX(10),FW_FIX(40),FW_FIX(300),FW_FIX(120)},
			FW_kFixedBounds,			// binding
			500,						// max chars
			9,							// attributes: kDrawBox + kAutoScroll
			FW_HELVETICA12, 
			""							// initial text
		), 
		FW_RStaticText
		(
			0,							// view id
			{ FW_FIX(10),FW_FIX(130),FW_FIX(300),FW_FIX(154)},	// LTRB
			FW_kFixedBounds,			// binding
			JAPANESE_FONT, 
			FW_RGB_WHITE,				// erase color
			"͂ĂF"		// nyuuryoku site kudasai
		),
		RJEditView						// edit view for Japanese
		(
			kJapaneseEditView,			// view id
			{ FW_FIX(10),FW_FIX(160),FW_FIX(300),FW_FIX(240)},
			FW_kFixedBounds,			// binding
			500,						// max chars
			0xC9,						// attributes: kDrawBox + kAutoScroll + kInlineInput + kTextServices
			JAPANESE_FONT, 
			RJStringData("")			// initial text
		)
	},						// End list of frame's subviews
	{ }						// no scroller
};

//========================================================================================
// resource FW_RFrameLayout(kOpTestDialog)
//========================================================================================

resource FW_RFrameLayout(kOpTestDialog)
{
	{FW_FIX(100),FW_FIX(100)},		// LayoutSize
	
	{						// Start list of frame's subviews
		FW_RStaticText
		(
			0,							// view id
			{ FW_FIX(20),FW_FIX(40),FW_FIX(140),FW_FIX(60)},
			FW_kFixedBounds,			// binding
			FW_NORMAL_FONT, 
			FW_RGB_WHITE,				// erase color <== NEW!
			"Enter a char index:"			
		),
		FW_RStaticText 
		(
			kStaticResultID,			// view id
			{ FW_FIX(20),FW_FIX(70),FW_FIX(200),FW_FIX(90)},
			FW_kFixedBounds,			// binding
			FW_NORMAL_FONT, 
			FW_RGB_WHITE,				// erase color <== NEW!
			""			
		),
		RMyEditView						// edit view to enter index (number)
		(
			kIndexEditID,				// view id
			{ FW_FIX(150),FW_FIX(40),FW_FIX(280),FW_FIX(60)},
			FW_kFixedBounds,			// binding
			10,							// max chars
			9,							// attributes
			FW_NORMAL_FONT, 
			""							// initial text
		), 
		FW_RButton
		(
			kOKButtonID,				// view id
			{ FW_FIX(180),FW_FIX(120),FW_FIX(260),FW_FIX(148)},
			FW_kFixedBounds,			// binding
			FW_kDefaultButtonMsg,		// "OK" message
			0,							// receiver = dialogFrame
			0,							// control value
			FW_kDefaultPushButton,		// button kind (with outline)
			FW_NORMAL_FONT,
			"OK"			
		),
		FW_RButton
		(
			kDoneButtonID,				// view id
			{ FW_FIX(60),FW_FIX(123),FW_FIX(120),FW_FIX(145)},
			FW_kFixedBounds,			// binding
			FW_kCancelButtonMsg,		// "Cancel" message
			0,							// receiver = dialogFrame by default
			0,							// control value
			FW_kPushButton,				// button kind
			FW_NORMAL_FONT,
			"Done"			
		)
	},						// End list of frame's subviews
	{ }						// no scroller
};

//========================================================================================
// resource FW_RFrameLayout(kReaderTestDialog)
//========================================================================================
resource FW_RFrameLayout(kReaderTestDialog)
{
	{FW_FIX(300),FW_FIX(200)},		// LayoutSize
	
	{						// Start list of frame's subviews
		FW_RStaticText 
		(
			0,							// view id
			{ FW_FIX(10),FW_FIX(20),FW_FIX(90),FW_FIX(50)},	// LTRB
			FW_kFixedBounds,			// binding
			FW_NORMAL_FONT, 
			FW_RGB_WHITE,				// erase color <== NEW!
			"Character:"			
		),
		FW_RStaticText 
		(
			kStaticResultID,			// view id
			{ FW_FIX(94),FW_FIX(20),FW_FIX(130),FW_FIX(50)},	// LTRB
			FW_kFixedBounds,			// binding
			FW_NORMAL_FONT, 
			FW_RGB_WHITE,				// erase color <== NEW!
			""			
		),
		FW_RButton
		(
			kDoneButtonID,				// view id
			{ FW_FIX(210),FW_FIX(30),FW_FIX(260),FW_FIX(58)},
			FW_kFixedBounds,			// binding
			FW_kDefaultButtonMsg,		// "OK" message
			0,							// receiver = dialogFrame by default
			0,							// control value
			FW_kDefaultPushButton,		// button kind (with outline)
			FW_NORMAL_FONT,
			"Done"			
		),
		FW_RStaticText 
		(
			0,							// view id
			{ FW_FIX(10),FW_FIX(60),FW_FIX(100),FW_FIX(80)},	// LTRB
			FW_kFixedBounds,			// binding
			FW_NORMAL_FONT, 
			FW_RGB_WHITE,				// erase color <== NEW!
			"Byte Position:"			
		),
		FW_RStaticText 
		(
			kBytePositionID,			// view id
			{ FW_FIX(104),FW_FIX(60),FW_FIX(130),FW_FIX(80)},	// LTRB
			FW_kFixedBounds,			// binding
			FW_NORMAL_FONT, 
			FW_RGB_WHITE,				// erase color <== NEW!
			""			
		),
		FW_RButton
		(
			kGetCharAdvButtonID,		// view id
			{ FW_FIX(10),FW_FIX(100),FW_FIX(150),FW_FIX(122)},
			FW_kFixedBounds,			// binding
			FW_kButtonPressedMsg,		// standard button message
			0,							// receiver = dialogFrame
			0,							// control value
			FW_kPushButton,				// button kind
			FW_NORMAL_FONT,
			"Get Char and Advance"			
		),
		FW_RButton
		(
			kBackupGetCharButtonID,		// view id
			{ FW_FIX(160),FW_FIX(100),FW_FIX(290),FW_FIX(122)},
			FW_kFixedBounds,			// binding
			FW_kButtonPressedMsg,		// standard button message
			0,							// receiver = dialogFrame
			0,							// control value
			FW_kPushButton,				// button kind
			FW_NORMAL_FONT,
			"Backup and Get Char"			
		),
		FW_RButton
		(
			kAdvanceButtonID,			// view id
			{ FW_FIX(10),FW_FIX(140),FW_FIX(90),FW_FIX(162)},	// LTRB
			FW_kFixedBounds,			// binding
			FW_kButtonPressedMsg,		// standard button message
			0,							// receiver = dialogFrame
			0,							// control value
			FW_kPushButton,				// button kind
			FW_NORMAL_FONT,
			"Advance"			
		),
		FW_RButton
		(
			kBackupButtonID,			// view id
			{ FW_FIX(110),FW_FIX(140),FW_FIX(190),FW_FIX(162)},
			FW_kFixedBounds,			// binding
			FW_kButtonPressedMsg,		// standard button message
			0,							// receiver = dialogFrame
			0,							// control value
			FW_kPushButton,				// button kind
			FW_NORMAL_FONT,
			"Backup"			
		),
		FW_RButton
		(
			kPeekCharButtonID,			// view id
			{ FW_FIX(210),FW_FIX(140),FW_FIX(290),FW_FIX(162)},
			FW_kFixedBounds,			// binding
			FW_kButtonPressedMsg,		// standard button message
			0,							// receiver = dialogFrame
			0,							// control value
			FW_kPushButton,				// button kind
			FW_NORMAL_FONT,
			"Peek Char"			
		)
	},						// End list of frame's subviews
	{ }						// no scroller
};

//========================================================================================
// resource FW_RFrameLayout(kNumberTestDialog)
//========================================================================================

resource FW_RFrameLayout(kNumberTestDialog)
{
	{FW_FIX(300),FW_FIX(160)},		// LayoutSize
	
	{	// Start list of frame's subviews
		FW_RStaticText
		(
			0,							// view id
			{ FW_FIX(20),FW_FIX(30),FW_FIX(140),FW_FIX(50)},
			FW_kFixedBounds,			// binding
			FW_NORMAL_FONT, 
			FW_RGB_WHITE,				// erase color <== NEW!
			"Enter a real number:"			
		),
		FW_RStaticText 
		(
			kStaticResultID,			// view id
			{ FW_FIX(20),FW_FIX(80),FW_FIX(200),FW_FIX(100)},
			FW_kFixedBounds,			// binding
			FW_NORMAL_FONT, 
			FW_RGB_WHITE,				// erase color <== NEW!
			""			
		),
		RMyEditView						// edit view to enter number
		(
			kNumberInputID,				// view id
			{ FW_FIX(150),FW_FIX(30),FW_FIX(280),FW_FIX(50)},
			FW_kFixedBounds,			// binding
			30,							// max chars
			9,							// attributes
			FW_NORMAL_FONT, 
			""							// initial text
		),
		FW_RButton
		(
			kOKButtonID,				// view id
			{ FW_FIX(180),FW_FIX(124),FW_FIX(260),FW_FIX(152)},
			FW_kFixedBounds,			// binding
			FW_kDefaultButtonMsg,		// "OK" message
			0,							// receiver = dialogFrame
			0,							// control value
			FW_kDefaultPushButton,		// button kind (with outline)
			FW_NORMAL_FONT,
			"OK"			
		),
		FW_RButton
		(
			kDoneButtonID,				// view id
			{ FW_FIX(60),FW_FIX(127),FW_FIX(120),FW_FIX(149)},
			FW_kFixedBounds,			// binding
			FW_kCancelButtonMsg,		// "Cancel" message
			0,							// receiver = dialogFrame by default
			0,							// control value
			FW_kPushButton,				// button kind
			FW_NORMAL_FONT,
			"Done"			
		),
		FW_RButton
		(
			kJCheckBoxID,				// view id
			{ FW_FIX(200),FW_FIX(60),FW_FIX(290),FW_FIX(80)},
			FW_kFixedBounds,			// binding
			FW_kButtonPressedMsg,		// standard button message
			0,							// receiver = dialogFrame
			0,							// control value
			FW_kCheckButton,			// button kind
			JAPANESE_FONT,
			"Japanese"			
		)
	},						// End list of frame's subviews
	{ }						// no scroller
};

